package com.exmachina.clients.geffen.smartwipe;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.DatastoreServiceFactory;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.FetchOptions;
import com.google.appengine.api.datastore.Query;
import com.google.appengine.api.datastore.Query.FilterOperator;
import com.google.appengine.api.datastore.Query.FilterPredicate;

@SuppressWarnings("serial")
public class SmartWipeServlet extends HttpServlet {

	private final static String REGISTER_ACTION = "register";
	private final static String UPDATE_ACTION = "update";
	private final static String QUERY_ACTION = "query";
	private final static String CLEAN_ACTION = "clean";

	private DatastoreService ds;

	@Override
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		ds = DatastoreServiceFactory.getDatastoreService();

		// Query Example
		// Query q = new Query("User");
		// Collection<Filter> filters = new ArrayList<Filter>();
		// filters.add(new FilterPredicate("uuid", FilterOperator.EQUAL, uuid));
		// CompositeFilter filter = new
		// CompositeFilter(CompositeFilterOperator.AND, filters);
		// q.setFilter(filter);
		// List<Entity> results =
		// ds.prepare(q).asList(FetchOptions.Builder.withDefaults());
		// if (results.size() == 1) {
		// Entity en = results.get(0);
		// en.setProperty("turns", turns);
		// ds.put(en);
		// }
	}

	public void service(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String action = req.getParameter("action");
		if (REGISTER_ACTION.equals(action)) {
			resp.setContentType("text/html");
			ServletOutputStream out = resp.getOutputStream();
			String name = req.getParameter("name");
			String phone = req.getParameter("phone");
			String uuid = req.getParameter("uuid");
			Entity user = null;
			if(uuidExists(uuid)) {
				user = getUserByUUID(uuid);
			}
			else {
				user = new Entity("User");
				user.setProperty("uuid", uuid);
			}
			user.setProperty("name", name);
			user.setProperty("phone", phone);
			ds.put(user);
			out.println("ok");
		}
		else if (UPDATE_ACTION.equals(action)) {
			resp.setContentType("text/html");
			ServletOutputStream out = resp.getOutputStream();
			String uuid = req.getParameter("uuid");
			String turns = req.getParameter("turns");
			if (!empty(uuid) && !empty(turns)) {
				if(uuidExists(uuid)) {
					if(isNumber(turns)) {
						Entity usage = new Entity("Usage");
						usage.setProperty("uuid", uuid);
						usage.setProperty("date", new Date());
						usage.setProperty("turns", Integer.parseInt(turns));
						ds.put(usage);
						out.println("ok");
					}
					else {
						out.println("bad request, parameter turns not a number");
					}
				}
				else {
					out.println("bad request, invalid uuid");
				}
			}
			else {
				out.println("bad request, mandatory parameters not specified: uuid, turns");
			}
		} 
		else if (QUERY_ACTION.equals(action)) {
			ServletOutputStream out = resp.getOutputStream();
			String uuid = req.getParameter("uuid");
			if(uuid == null || uuid.trim().length() == 0) {
				resp.setContentType("text/html");
				out.println("<html><head><title>SmartWipe Console</title></head><body>");
	
				out.println("<h2>Users</h2>");
				out.println("<table border=\"1\">");
				Query q = new Query("User");
				List<Entity> results = ds.prepare(q).asList(FetchOptions.Builder.withDefaults());
				out.println("<tr><td>Name</td><td>Phone</td><td>UUID</td><td>Turns</td></tr>");
				for (Entity e : results) {
					out.println("<tr>");
					out.println("<td>" + e.getProperty("name") + "</td>");
					out.println("<td>" + e.getProperty("phone") + "</td>");
					out.println("<td>" + e.getProperty("uuid") + "</td>");
					out.println("<td>" + e.getProperty("turns") + "</td>");
					out.println("</tr>");
				}
				out.println("</table>");
	
				out.println("<h2>Usage</h2>");
				out.println("<table border=\"1\">");
				Query q2 = new Query("Usage");
				List<Entity> results2 = ds.prepare(q2).asList(FetchOptions.Builder.withDefaults());
				out.println("<tr><td>UUID</td><td>Date</td><td>Turns</td></tr>");
				for (Entity e : results2) {
					out.println("<tr>");
					out.println("<td>" + e.getProperty("uuid") + "</td>");
					out.println("<td>" + e.getProperty("date") + "</td>");
					out.println("<td>" + e.getProperty("turns") + "</td>");
					out.println("</tr>");
				}
				out.println("</table>");
				out.println("</body></html>");
			}
			else {
				resp.setContentType("application/json");
				Query q2 = new Query("Usage");
				q2.setFilter(new FilterPredicate("uuid", FilterOperator.EQUAL, uuid));
				List<Entity> results2 = ds.prepare(q2).asList(FetchOptions.Builder.withDefaults());
				out.println("{ \"entries\": [");
				int counter = 0;
				for (Entity e : results2) {
					out.println("{");
					out.println("\"uuid\": \"" + e.getProperty("uuid") + "\", ");
					out.println("\"date\": \"" + e.getProperty("date") + "\", ");
					out.println("\"turns\": \"" + e.getProperty("turns") + "\"");
					out.println("}");
					if(counter < (results2.size() - 1)) {
						out.println(",");
					}
				
				}
				out.println("]}");
				
			}
		}
		else if (CLEAN_ACTION.equals(action)) {
			resp.setContentType("text/html");
			ServletOutputStream out = resp.getOutputStream();
			Query q = new Query("User");
			List<Entity> results = ds.prepare(q).asList(FetchOptions.Builder.withDefaults());
			for (Entity e : results) {
				ds.delete(e.getKey());
			}
			Query q2 = new Query("Usage");
			List<Entity> results2 = ds.prepare(q2).asList(FetchOptions.Builder.withDefaults());
			for (Entity e : results2) {
				ds.delete(e.getKey());
			}
			out.println("ok");
		}		
		else {
			resp.setContentType("text/html");
			ServletOutputStream out = resp.getOutputStream();
			out.println("no action");
		}
	}

	private boolean uuidExists(String uuid) {
		Query q = new Query("User");
		q.setFilter(new FilterPredicate("uuid", FilterOperator.EQUAL, uuid));
		List<Entity> results = ds.prepare(q).asList(FetchOptions.Builder.withDefaults());
		return (results.size() > 0);
	}

	private Entity getUserByUUID(String uuid) {
		Query q = new Query("User");
		q.setFilter(new FilterPredicate("uuid", FilterOperator.EQUAL, uuid));
		List<Entity> results = ds.prepare(q).asList(FetchOptions.Builder.withDefaults());
		if (results.size() > 0) {
			return results.get(0);
		} else
			return null;
	}

	private boolean empty(String str) {
		return (str == null) || (str.trim().length() == 0);
	}

	private boolean isNumber(String s) {
		try {
			Integer.parseInt(s);
			return true;
		} catch (Throwable th) {
			return false;
		}
	}

}
